@@ -7,10 +7,10 @@ import scipy.stats
 
 from superwise.resources.superwise_enums import BooleanSecondaryType
 from superwise.resources.superwise_enums import CategoricalSecondaryType
-from superwise.resources.superwise_enums import DataTypesRoles
+from superwise.resources.superwise_enums import DataEntityRole
 from superwise.resources.superwise_enums import FeatureType
+from superwise.resources.superwise_enums import ModelTypes
 from superwise.resources.superwise_enums import NumericSecondaryType
-from superwise.resources.superwise_enums import TaskTypes
 
 logger = logging.getLogger(__name__)
 
@@ -45,7 +45,7 @@ class EntitiesValidationError(RuntimeError):
 
 
 class EntitiesValidator:
-    def __init__(self, task, version_entities_df, data, MAX_DIMENSIONS=None):
+    def __init__(self, version_entities_df, data, MAX_DIMENSIONS=None):
         """
             version_entities_df
                 name | type | role | is_dimension
@@ -56,7 +56,6 @@ class EntitiesValidator:
         """
         self.MAX_DIMENSIONS = MAX_DIMENSIONS or int(os.environ.get("MAX_DIMENSIONS", 10))
         self._version_entities_df = version_entities_df
-        self._task = task
         self._data = data
 
     def _assert_type(self, column, column_description, expected_type):
@@ -85,6 +84,13 @@ class EntitiesValidator:
             error_msg = "Column {} ({}) has null values".format(column, column_description)
             raise EntitiesValidationError(error_msg)
 
+    @staticmethod
+    def _assert_not_inf(data, column, column_description):
+        cnt = data[column].isin([np.inf, -np.inf]).values.sum()
+        if cnt:
+            error_msg = f"Column {column} ({column_description}) has {cnt} records with inf/-inf values"
+            raise EntitiesValidationError(error_msg)
+
     @staticmethod
     def _assert_not_empty(data, column, column_description):
         if data[column].isna().all():
@@ -117,7 +123,7 @@ class EntitiesValidator:
             raise EntitiesValidationError(error_msg)
 
     def _is_valid_label_weight(self):
-        label_weight_col = self._version_entities_df["role"] == DataTypesRoles.LABEL_WEIGHT.value
+        label_weight_col = self._version_entities_df["role"] == DataEntityRole.LABEL_WEIGHT.value
         label_weight_size = label_weight_col.sum()
         if label_weight_size > 1:
             raise EntitiesValidationError("more then one ({}) label weight entity".format(label_weight_size))
@@ -139,7 +145,7 @@ class EntitiesValidator:
 
     def _has_all_required_roles(self):
         existing_role = set(self._version_entities_df["role"].unique())
-        required_roles = set([DataTypesRoles.ID.value, DataTypesRoles.TIMESTAMP.value])
+        required_roles = set([DataEntityRole.ID.value, DataEntityRole.TIMESTAMP.value])
         self._assert_subset(required_roles, "required roles", existing_role, "existing roles")
 
     def _column_names_match_version_entities(self):
@@ -162,36 +168,6 @@ class EntitiesValidator:
         for dimension in dimensions:
             self._assert_type(dimension, "dimension", FeatureType.CATEGORICAL.value)
 
-    def _is_valid_label(self):
-        label_col = self._version_entities_df["role"] == DataTypesRoles.LABEL.value
-        if self._task.task_type == TaskTypes.REGRESSION.value:
-            invalid_feature_type_col = self._version_entities_df[label_col]["type"] != FeatureType.NUMERIC.value
-        else:
-            invalid_feature_type_col = (
-                self._version_entities_df[label_col]["type"].isin(
-                    [FeatureType.CATEGORICAL.value, FeatureType.BOOLEAN.value]
-                )
-                == False
-            )
-        if len(self._version_entities_df[label_col][invalid_feature_type_col]) > 0:
-            raise Exception("invalid feature type")
-        return True
-
-    def _is_valid_prediction_value(self):
-        pred_val_col = self._version_entities_df["role"] == DataTypesRoles.PREDICTION_VALUE.value
-        if self._task.task_type == TaskTypes.REGRESSION.value:
-            invalid_feature_type_col = self._version_entities_df[pred_val_col]["type"] != FeatureType.NUMERIC.value
-        else:
-            invalid_feature_type_col = (
-                self._version_entities_df[pred_val_col]["type"].isin(
-                    [FeatureType.CATEGORICAL.value, FeatureType.BOOLEAN.value]
-                )
-                == False
-            )
-        if len(self._version_entities_df[pred_val_col][invalid_feature_type_col]) > 0:
-            raise Exception("invalid feature type")
-        return True
-
     def _is_valid_prediction_probability(self):
         columns = self._version_entities_df[self._version_entities_df["role"] == "prediction probability"]
         for idx, column in columns.iterrows():
@@ -236,52 +212,27 @@ class EntitiesValidator:
         for boolean_column in boolean_columns:
             self._assert_not_empty(self._data, boolean_column, "boolean")
 
-            if self._data[boolean_column].dtype == np.dtype(object):
-                self._data[boolean_column] = self._data[boolean_column].str.lower()
-
             unique_values = set(self._data[boolean_column].dropna().unique())
             if len(unique_values) == 0:
                 error_msg = "Boolean column {} is empty".format(boolean_column)
                 raise EntitiesValidationError(error_msg)
 
-            if not (
-                unique_values <= {True, False}
-                or unique_values <= {1, 0}
-                or unique_values <= {1.0, 0.0}
-                or unique_values <= {"true", "false"}
-                or unique_values <= {"1", "0"}
-                or unique_values <= {"1.0", "0.0"}
-            ):
+            if not (unique_values <= {True, False} or unique_values <= {1, 0}):
                 error_msg = (
                     "Boolean column must be either a subset of {{True, False}} or {{1, 0}}. "
                     "You provided {} for column {}".format(",".join(map(str, unique_values)), boolean_column)
                 )
                 raise EntitiesValidationError(error_msg)
-
-            if self._data[boolean_column].dtype == np.dtype(object):
-                self._data[boolean_column] = self._data[boolean_column].map(
-                    {"true": True, "false": False, "1": True, "0": False, "1.0": True, "0.0": False}
-                )
-            else:
-                self._data[boolean_column] = self._data[boolean_column].astype("boolean")
+            self._data[boolean_column] = self._data[boolean_column].astype("boolean")
 
     def _numeric_types_valid(self):
         numeric_columns = self._version_entities_df[self._version_entities_df["type"] == FeatureType.NUMERIC.value][
             "name"
         ].tolist()
         for numeric_column in numeric_columns:
+            self._assert_not_inf(self._data, numeric_column, "numeric")
             self._assert_not_empty(self._data, numeric_column, "numeric")
-            try:
-                self._data[numeric_column] = pd.to_numeric(self._data[numeric_column])
-                if numeric_column not in ["label_weight"]:
-                    try:
-                        self._data[numeric_column] = self._data[numeric_column].astype("Int64")
-                    except TypeError:
-                        pass
-                    except:
-                        pass
-            except:
-                raise EntitiesValidationError("cast column {}  to numeric failed".format(numeric_column))
+            self._data[numeric_column] = self._data[numeric_column].astype("float")
 
     def _categorical_types_valid(self):
         categorical_columns = self._version_entities_df[
@@ -297,11 +248,7 @@ class EntitiesValidator:
             feature_data = data[feature_name].dropna()
             feature_stat = dict()
             feature_stat["name"] = feature_name
-            feature_stat["pandas_type"] = (
-                data[feature_name].dtype.type
-                if isinstance(data[feature_name].dtype, pd.Int64Dtype)
-                else data[feature_name].dtype
-            )
+            feature_stat["pandas_type"] = data[feature_name].dtype
             hist = feature_data.value_counts(normalize=True)
             feature_stat["num_unique"] = len(hist)
             feature_stat["% unique"] = len(hist) / len(data)
@@ -340,6 +287,42 @@ class EntitiesValidator:
         elif self._version_entities_df["secondary_type"].isnull().values.any():
             raise EntitiesValidationError("seconday_type supplied for some entities, should be supplied for all or non")
 
+    def infer_data_type(self):
+        logger.info("Infer parquet type based on type")
+        logger.debug("Type map -> Unknown=string, numeric=float64, timestamp=timestamp, boolean=bool_")
+        data_types = {
+            FeatureType.UNKNOWN.value: "text",
+            FeatureType.NUMERIC.value: "number",
+            FeatureType.TIMESTAMP.value: "timestamp",
+            FeatureType.BOOLEAN.value: "bool",
+        }
+        self._version_entities_df["data_type"] = self._version_entities_df["type"].map(data_types)
+        logger.info("Infer parquet type for categorical based on data")
+        for feature_name in self._version_entities_df.loc[
+            self._version_entities_df["type"] == FeatureType.CATEGORICAL.value, "name"
+        ].tolist():
+            try:
+                self._data[feature_name].astype("float")
+                self._version_entities_df.loc[self._version_entities_df["name"] == feature_name, "data_type"] = "number"
+                logger.debug(f"Categorical feature {feature_name} has parquet type float64")
+            except:
+                self._version_entities_df.loc[self._version_entities_df["name"] == feature_name, "data_type"] = "text"
+                logger.debug(f"Categorical feature {feature_name} has parquet type string")
+
+    def convert_to_data_type(self):
+        logger.info("Convert to inferred data type")
+        data_type_to_conversion = {
+            "text": lambda col: col.astype("string"),
+            "number": lambda col: col.astype("float"),
+            "bool": lambda col: col.astype("boolean"),
+            "timestamp": lambda col: pd.to_datetime(col, format="%Y-%m-%d %H:%M:%S"),
+        }
+        for version_entity in self._version_entities_df.to_dict(orient="records"):
+            feature_name = version_entity["name"]
+            data_type = version_entity["data_type"]
+            self._data[feature_name] = data_type_to_conversion[data_type](self._data[feature_name])
+        logger.info("Conversion to inferred data type finished")
+
     def prepare(self):
         """
         Validate and prepare the data
@@ -368,10 +351,10 @@ class EntitiesValidator:
         self._has_all_required_roles()
         self._is_valid_record_id()
         self._is_valid_prediction_ts()
-        self._is_valid_prediction_value()
         self._is_valid_prediction_probability()
-        self._is_valid_label()
         self._is_valid_label_ts()
         self._is_valid_label_weight()
         self.infer_secondary_type()
+        self.infer_data_type()
+        self.convert_to_data_type()
         return self._data
