@@ -94,17 +94,22 @@ class FeatureImportance:
             return res.iloc[0]["name"]
         return None
 
-    def _calc(self, baseline_data):
+    def _calc(self, baseline_data, target=None, sample=None):
         logger.debug("calc feature importance")
+        if sample:
+            baseline_data = baseline_data.sample(frac=sample)
+
         processed_features = self._fi_pre_processing(baseline_data, self)
         reconstruction_model = RandomForestRegressor(max_depth=7, n_estimators=50)
         label_name = None
-        if not label_name:
+        if target:
+            label_name = target
+        else:
             label_name = self.get_label_by_role("prediction value")
-        if not label_name:
-            label_name = self.get_label_by_role("label")
-        if not label_name:
-            raise Exception("no valid label to fit model")
+            if not label_name:
+                label_name = self.get_label_by_role("label")
+            if not label_name:
+                raise Exception("no valid label to fit model")
 
         label = baseline_data[label_name]
         if not pd.api.types.is_numeric_dtype(label):
@@ -120,10 +125,10 @@ class FeatureImportance:
         importance = dict(zip(processed_features.columns, mean_shap.tolist()))
         return importance
 
-    def compute(self, data):
+    def compute(self, data, target=None, sample=None):
         if "feature_importance" not in self._entities_df or pd.isnull(self._entities_df["feature_importance"]).all():
             logger.info("Feature importance was NOT provided as part of the version entities. Computing")
-            feature_name_to_importance = self._calc(data)
+            feature_name_to_importance = self._calc(data, target, sample)
             self._entities_df["feature_importance"] = self._entities_df["name"].map(feature_name_to_importance)
         else:
             logger.info("Feature importance was provided as part of the version entities. Skipping Computation")
