@@ -3,6 +3,7 @@ import logging
 import numpy as np
 import pandas as pd
 from sklearn.linear_model import LinearRegression
+from sklearn.preprocessing import KBinsDiscretizer
 
 from .entity_summary_generator import EntitySummaryGenerator
 from superwise.resources.superwise_enums import NumericSecondaryType
@@ -14,52 +15,15 @@ class NumericalSummaryGenerator(EntitySummaryGenerator):
     def __init__(self, entity):
         super(NumericalSummaryGenerator, self).__init__(entity)
 
-    @classmethod
-    def _head_tail_break_right(cls, data, breaks):
-        n = len(data)
-        m = data.mean()
-        head = data[data > m]
-        breaks.append(m)
-        if len(head) < 0.4 * n:
-            return cls._head_tail_break_right(head, breaks)
-        else:
-            return breaks
-
-    @classmethod
-    def _head_tail_break_left(cls, data, breaks):
-        n = len(data)
-        m = data.mean()
-        tail = data[data < m]
-        breaks.append(m)
-        if len(tail) < 0.4 * n:
-            return cls._head_tail_break_left(tail, breaks)
-        else:
-            return breaks
-
-    def _long_tail_and_quantiles_bin(self, secondary_type):
-        _, quantiles = pd.qcut(self._entity, 10, retbins=True, duplicates="drop")
-        if secondary_type == NumericSecondaryType.NUM_RIGHT_TAIL.value:
-            head_tail = np.array(self._head_tail_break_right(self._entity, breaks=[]))
-            head_tail = head_tail[~pd.isna(head_tail)]
-            head_tail.sort()
-            return np.append(quantiles[quantiles < head_tail.min()], head_tail[1:])
-        elif secondary_type == NumericSecondaryType.NUM_LEFT_TAIL.value:
-            head_tail = np.array(self._head_tail_break_left(self._entity, breaks=[]))
-            head_tail = head_tail[~pd.isna(head_tail)]
-            head_tail.sort()
-            return np.append(head_tail[:-1], quantiles[quantiles > head_tail.max()])
-        else:
-            return quantiles
-
     def _get_linear_percentile(self, quantile_deviation=0.03):
         # upper limit
         q_values = self._entity.astype(float).quantile(q=[0.98, 0.985, 0.99, 0.995])
-        lr = LinearRegression().fit(q_values.reset_index()[["index"]], q_values.values)
+        lr = LinearRegression().fit(q_values.reset_index()[["index"]].values, q_values.values)
         q_predict = 1 + quantile_deviation
         upper_limit = float(lr.predict(np.array(q_predict).reshape(-1, 1))[0])
         # Lower limit
         q_values = self._entity.astype(float).quantile(q=[0.005, 0.01, 0.015, 0.02])
-        lr = LinearRegression().fit(q_values.reset_index()[["index"]], q_values.values)
+        lr = LinearRegression().fit(q_values.reset_index()[["index"]].values, q_values.values)
         q_predict = 0 - quantile_deviation
         lower_limit = float(lr.predict(np.array(q_predict).reshape(-1, 1))[0])
         return lower_limit, upper_limit
@@ -101,26 +65,26 @@ class NumericalSummaryGenerator(EntitySummaryGenerator):
         mean_value = None if pd.isna(mean_value) else float(mean_value)
         return {"statistics": {"mean": mean_value}}
 
-    def calc_distribution(self, secondary_type):
-        return {
-            NumericSecondaryType.NUM_RIGHT_TAIL.value: self.calc_right_tailed_distribution,
-            NumericSecondaryType.NUM_LEFT_TAIL.value: self.calc_left_tailed_distribution,
-            NumericSecondaryType.NUM_CENTERED.value: self.calc_centered_distribution,
-        }[secondary_type]
+    def _numeric_bins(self, n_bins=30, strategy="kmeans", pad_bins=2):
+        nunique = self._entity.nunique()
+        if nunique == 1:
+            return [self._entity.dropna().iloc[0]]
+        n_bins = min(nunique, n_bins)
+        discretizer = KBinsDiscretizer(n_bins=n_bins, encode="ordinal", strategy=strategy)
+        bins = discretizer.fit(self._entity.dropna().to_frame()).bin_edges_[0]
 
-    def calc_right_tailed_distribution(self):
-        logger.debug(f"Calc right tail distribution for {self._entity.name}")
-        bins = self._long_tail_and_quantiles_bin(NumericSecondaryType.NUM_RIGHT_TAIL.value)
-        return self._calc_distribution(bins)
+        if pad_bins:
+            lower_bin_width = bins[1] - bins[0]
+            lower_bins_padding = [bins[-1] + lower_bin_width * (i + 1) for i in range(pad_bins)]
+            upper_bin_width = bins[-1] - bins[-2]
+            upper_bins_padding = [bins[0] - upper_bin_width * (i + 1) for i in range(pad_bins)]
+            bins = np.append(bins, lower_bins_padding + upper_bins_padding)
+            bins.sort()
+        return bins
 
-    def calc_left_tailed_distribution(self):
-        logger.debug(f"Calc left tail distribution for {self._entity.name}")
-        bins = self._long_tail_and_quantiles_bin(NumericSecondaryType.NUM_LEFT_TAIL.value)
-        return self._calc_distribution(bins)
-
-    def calc_centered_distribution(self):
-        logger.debug(f"Calc centered distribution for {self._entity.name}")
-        bins = self._long_tail_and_quantiles_bin(NumericSecondaryType.NUM_CENTERED.value)
+    def calc_numeric_distribution(self):
+        logger.debug(f"Calc numeric distribution for {self._entity.name}")
+        bins = self._numeric_bins()
         return self._calc_distribution(bins)
 
     def _calc_distribution(self, bins):
@@ -166,7 +130,7 @@ class NumericalSummaryGenerator(EntitySummaryGenerator):
                 self.calc_min_value,
                 self.calc_max_value,
                 self.calc_mean_value,
-                self.calc_distribution(secondary_type),
+                self.calc_numeric_distribution,
                 self.calc_std,
             ],
             NumericSecondaryType.NUM_LEFT_TAIL.value: [
@@ -176,7 +140,7 @@ class NumericalSummaryGenerator(EntitySummaryGenerator):
                 self.calc_max_value,
                 self.calc_mean_value,
                 self.calc_std,
-                self.calc_distribution(secondary_type),
+                self.calc_numeric_distribution,
             ],
             NumericSecondaryType.NUM_CENTERED.value: [
                 self.calc_missing_values,
@@ -185,7 +149,7 @@ class NumericalSummaryGenerator(EntitySummaryGenerator):
                 self.calc_max_value,
                 self.calc_mean_value,
                 self.calc_std,
-                self.calc_distribution(secondary_type),
+                self.calc_numeric_distribution,
             ],
         }
         logger.debug(f"Generate summary for Numerical ({secondary_type}) feature {self._entity.name}")
