@@ -9,9 +9,29 @@ class BaseModel:
     """ Base model """
 
     def __init__(self, **kwargs):
+        """
+         ### Description:
+
+         Constructor of BaseModel class
+
+         """
         self.id = kwargs("id", None)
 
     def from_datetime(self, date):
+        """
+         ### Description:
+
+         Convert datetime helper
+
+         ### Args:
+
+         `date`: a datetime.datetime object
+
+        ### Return:
+
+        string of date in iso format
+        """
+
         if pd.isna(date):
             return None
         elif isinstance(date, datetime.datetime):
@@ -20,20 +40,21 @@ class BaseModel:
             return date
 
     def get_properties(self):
-
         """
-        get properrties of model as dictionary
+        ### Description:
 
-        :return: properties dict
+        Get properties of model as a dictionary
+
+        ### Args:
+
+        `date`: a datetime.datetime object
+
+        ### Return:
+
+        dictionary of model properties
         """
         return dict(
             (name, getattr(self, name))
             for name in dir(self)
             if not name.startswith("__") and not callable(getattr(self, name))
         )
-
-    def get_enum_value(self, v):
-        if isinstance(v, Enum):
-            return v.value
-        else:
-            return v
