@@ -4,6 +4,7 @@ import json
 import pandas as pd
 
 from superwise.models.base import BaseModel
+from superwise.resources.superwise_enums import get_enum_value
 
 
 class DataEntity(BaseModel):
@@ -19,58 +20,104 @@ class DataEntity(BaseModel):
         feature_importance=None,
         summary=None,
         secondary_type=None,
+        data_type=None,
         **kwargs
     ):
         """
-        constructer for DataEntity class
+        ### Description:
 
-        :param type:
-        :param name:
-        :param is_dimension:
-        :param role:
-        :param feature_importance:
+        Constructor for DataEntity class
+
+        ### Args:
+
+        `id`: id if dataentity
+
+        `dimension_start_ts`:
+
+        `type`:
+
+        `name`:
+
+        `is_dimension`:
+
+        `role`:
+
+        `feature_importance`:
+
+        `summary`: dictionary of summarised data
+
+        `secondary_type`:
+
+        `data_type`:
         """
+
         self.name = name.lower() if name else None
-        self.type = self.get_enum_value(type)
-        self.role = self.get_enum_value(role)
+        self.type = get_enum_value(type)
+        self.role = get_enum_value(role)
         self.feature_importance = feature_importance
         self.summary = summary
         self.secondary_type = secondary_type
         self.id = id
         self.dimension_start_ts = self.from_datetime(dimension_start_ts)
+        self.data_type = data_type
 
     @staticmethod
     def list_to_df(data_entities):
+        """
+        ### Description:
+
+        Get list of DataEntity objects and return them as pandas dataframe
+
+        ### Args:
+
+        `data_entities`: List[DataEntity] - list of DataEntity objects
+
+        ### Return:
+
+        pandas df of data entities
+         """
+
         data = [d.get_properties() for d in data_entities]
         df = pd.DataFrame(data)
         return df
 
     @staticmethod
     def df_to_list(data_entities_df):
+        """
+        ### Description:
+
+        Get data entities dataframe and return list of DataEntity objects
+
+        ### Args:
+
+        `data_entities_df`: pandas df of data entities
+
+        ### Return:
+
+        List[DataEntity] - list of DataEntity objects
+         """
+
         data = json.loads(data_entities_df.to_json(orient="records"))
         entities = []
         [entities.append(DataEntity(**d)) for d in data]
         return entities
 
 
-class DataEntityCollection:
-    def __init__(self, entities=None, reuse_previus_entities=False):
-        self.reuse_previus_entities = reuse_previus_entities
-        self.entities = entities or []
-
-    def add_entity(self, entity):
-        self.entities.append(entity)
-
-
 class DataEntitySummary(BaseModel):
     """ summary model class """
 
     def __init__(self, idx=None, summary=None):
         """
-        constructer for DataEntitySummary class
+        ### Description:
 
-        :param idx:
-        :param summary:
+        Constructer for DataEntitySummary class
+
+        ### Args:
+
+        `idx`: id of dataEntity
+
+        `summary`: summary of dataentity (dict)
         """
+
         self.id = idx
         self.summary = summary
