@@ -8,7 +8,7 @@ class Role(BaseModel):
     def __init__(
         self,
         id=None,
-        task_type_id=None,
+        model_type_id=None,
         is_nullable=None,
         description=None,
         internal_name=None,
@@ -19,20 +19,32 @@ class Role(BaseModel):
         **kwargs
     ):
         """
-        constructer for Role class
+        ### Description:
+
+        Constructor of Role class
+
+        ### Args:
+
+        `id`: id of role (int)
+
+        `model_type_id`: model type id (ModelType enum value)
+
+        `is_nullable`:
+
+        `description`: description of the role
+
+         internal_name`:
+
+        `is_label`:
+
+        `is_optional`:
+
+        `role`:
+
+        `value`:
 
-        :param id:
-        :param task_type_id:
-        :param is_nullable:
-        :param task_description:
-        :param description:
-        :param internal_name:
-        :param is_label:
-        :param is_optional:
-        :param role:
-        :param value:
         """
-        self.task_type_id = task_type_id
+        self.model_type_id = model_type_id
         self.description = description
         self.internal_name = internal_name
         self.is_label = is_label
