@@ -4,14 +4,21 @@ from superwise.models.base import BaseModel
 
 
 class ValidationError(BaseModel):
-    """ validation error model class, model  for validation errors """
+    """ validation error model class """
 
     def __init__(self, http_status_code=None, http_error_reason=None, body=None):
         """
+        ### Description:
 
-        :param http_status_code: http status code from server
-        :param http_error_reason: reason for error
-        :param body:jbody of response object
+        Constructor for ValidationError, this class used for handling validation errors
+
+        ### Args:
+
+         `http_status_code`: http status code from server
+
+         `http_error_reason`: reason for error
+
+         `body`: body of response object
         """
         self.http_status_code = http_status_code
         self.http_error_reason = http_error_reason
